import {
  Body,
  Controller,
  Get,
  Post,
  Headers,
  NotFoundException,
} from '@nestjs/common';
import { ProductsService } from './products.service';
import { StripeService } from '../stripe/stripe.service';
import { PreOrdersService } from '../orders/pre-orders.service';

@Controller('products')
export class ProductsController {
  constructor(
    private productsService: ProductsService,
    private stripeService: StripeService,
    private preOrdersService: PreOrdersService,
  ) {}

  @Get('')
  getAll() {
    return this.productsService.getAll();
  }

  @Post('checkout')
  async Checkout(@Body('cartItems') cartItems, @Headers() headers) {
    const products = await this.productsService.getProducts(
      cartItems.map((t) => t.id),
    );
    if (products && products.length > 0) {
      const itemsArr = products.map((product) => ({
        id: product.id,
        name: product.title,
        price: product.price,
        imageUrl: product.imageUrl,
        quantity:
          cartItems.find((item) => item.id === product.id)?.quantity ?? 1,
      }));
      let total = 0;
      const userId = 1;
      itemsArr.forEach((item) => {
        total += Math.round(item.price * item.quantity);
      });
      const preOrder = await this.preOrdersService.create({
        items: JSON.stringify(itemsArr),
        orderDate: new Date(),
        total,
        userId,
      });
      return this.stripeService.checkout(preOrder, headers);
    } else throw new NotFoundException('Products not found!');
  }
}
